@@ -119,6 +119,7 @@ typedef struct {
     int	num_init_children;	/* # of children initially pre-forked */
     int	child_life_time;	/* if idle for this seconds, child exits */
     int	connection_life_time;	/* if idle for this seconds, connection closes */
+    int	child_max_connections;	/* if max_connections received, child exits */
     int	max_pool;	/* max # of connection pool per child */
     char *logdir;		/* logging directory */
     char *backend_socket_dir;	/* Unix domain socket directory for the PostgreSQL server */
@@ -148,6 +149,7 @@ typedef struct {
 	char *health_check_user;		/* PostgreSQL user name for health check */
 	int insert_lock;	/* if non 0, automatically lock table with INSERT to keep SERIAL
 						   data consistency */
+	int ignore_leading_white_space;		/* ignore leading white spaces of each query */
 	/* followings do not exist in the configuration file */
     char *current_backend_host_name;	/* current backend host name */
     int	current_backend_port;	/* current backend port # */
