@@ -45,6 +45,14 @@ static POOL_STATUS NotificationResponse(POOL_CONNECTION *frontend,
 static POOL_STATUS Query(POOL_CONNECTION *frontend, 
 						 POOL_CONNECTION_POOL *backend, char *query);
 
+static POOL_STATUS Execute(POOL_CONNECTION *frontend, 
+						   POOL_CONNECTION_POOL *backend);
+
+#ifdef NOT_USED
+static POOL_STATUS Sync(POOL_CONNECTION *frontend, 
+						   POOL_CONNECTION_POOL *backend);
+#endif
+
 static POOL_STATUS ReadyForQuery(POOL_CONNECTION *frontend, 
 								 POOL_CONNECTION_POOL *backend, int send_ready);
 
@@ -650,6 +658,123 @@ static POOL_STATUS Query(POOL_CONNECTION *frontend,
 	return POOL_CONTINUE;
 }
 
+/*
+ * process EXECUTE (V3 only)
+ */
+static POOL_STATUS Execute(POOL_CONNECTION *frontend, 
+						   POOL_CONNECTION_POOL *backend)
+{
+	char *string;		/* portal name + null terminate + max_tobe_returned_rows */
+	int len;
+	int sendlen;
+	int i;
+
+	/* read Execute packet */
+	if (pool_read(frontend, &len, sizeof(len)) < 0)
+		return POOL_END;
+
+	len = ntohl(len) - 4;
+	string = pool_read2(frontend, len);
+
+	pool_debug("Execute: portal name <%s>", string);
+
+	for (i = 0;i < backend->num;i++)
+	{
+		POOL_CONNECTION *cp = backend->slots[i]->con;
+
+		/* forward the query to the backend */
+		pool_write(cp, "E", 1);
+		sendlen = htonl(len + 4);
+		pool_write(cp, &sendlen, sizeof(sendlen));
+		pool_write(cp, string, len);
+
+		/*
+		 * send "Flush" message so that backend notices us
+		 * the completion of the command
+		 */
+		pool_write(cp, "H", 1);
+		sendlen = htonl(4);
+		if (pool_write_and_flush(cp, &sendlen, sizeof(sendlen)) < 0)
+		{
+			return POOL_END;
+		}
+
+		if (!REPLICATION)
+			break;
+
+		/*
+		 * in "strict mode" we need to wait for backend completing the query.
+		 */
+		if (pool_config.replication_strict)
+		{
+			pool_debug("waiting for backend[%d] completing the query", i);
+			if (synchronize(cp))
+				return POOL_END;
+		}
+
+	}
+
+	return POOL_CONTINUE;
+}
+
+#ifdef NOT_USED
+/*
+ * process Sync (V3 only)
+ */
+static POOL_STATUS Sync(POOL_CONNECTION *frontend, 
+						   POOL_CONNECTION_POOL *backend)
+{
+	char *string;		/* portal name + null terminate + max_tobe_returned_rows */
+	int len;
+	int sendlen;
+
+	/* read Sync packet */
+	if (pool_read(frontend, &len, sizeof(len)) < 0)
+		return POOL_END;
+
+	len = ntohl(len) - 4;
+	string = pool_read2(frontend, len);
+
+	/* forward the query to the backend */
+	pool_write(MASTER(backend), "S", 1);
+
+	sendlen = htonl(len + 4);
+	pool_write(MASTER(backend), &sendlen, sizeof(sendlen));
+	if (pool_write_and_flush(MASTER(backend), string, len) < 0)
+	{
+		return POOL_END;
+	}
+
+	if (REPLICATION)
+	{
+		/*
+		 * in "strict mode" we need to wait for master completing the query.
+		 * note that this is not applied if "NO STRICT" is specified as a comment.
+		 */
+		if (pool_config.replication_strict)
+		{
+			pool_debug("waiting for master completing the query");
+			if (synchronize(MASTER(backend)))
+				return POOL_END;
+		}
+
+		pool_write(SECONDARY(backend), "S", 1);
+		sendlen = htonl(len + 4);
+		pool_write(SECONDARY(backend), &sendlen, sizeof(sendlen));
+		if (pool_write_and_flush(SECONDARY(backend), string, len) < 0)
+		{
+			return POOL_END;
+		}
+
+		/* in "strict mode" we need to wait for secondary completing the query */
+		if (pool_config.replication_strict)
+			if (synchronize(SECONDARY(backend)))
+				return POOL_END;
+	}
+	return POOL_CONTINUE;
+}
+#endif
+
 static POOL_STATUS ReadyForQuery(POOL_CONNECTION *frontend, 
 								 POOL_CONNECTION_POOL *backend, int send_ready)
 {
@@ -1513,6 +1638,16 @@ static POOL_STATUS ProcessFrontendResponse(POOL_CONNECTION *frontend,
 			status = Query(frontend, backend, NULL);
 			break;
 
+/*
+		case 'S':
+			status = Sync(frontend, backend);
+			break;
+*/
+
+		case 'E':
+			status = Execute(frontend, backend);
+		break;
+
 		default:
 			if (MAJOR(backend) == PROTO_MAJOR_V3)
 			{
@@ -1708,6 +1843,11 @@ static void process_reporting(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *b
 	status[i].desc = "if idle for this seconds, connection closes";
 	i++;
 
+	status[i].name = "child_max_connections";
+	snprintf(status[i].value, MAXVALLEN, "%d", pool_config.child_max_connections);
+	status[i].desc = "if max_connections received, chile exits";
+	i++;
+
 	status[i].name = "max_pool";
 	snprintf(status[i].value, MAXVALLEN, "%d", pool_config.max_pool);
 	status[i].desc = "max # of connection pool per child";
@@ -1806,6 +1946,11 @@ static void process_reporting(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *b
 	status[i].desc = "insert lock";
 	i++;
 
+	status[i].name = "ignore_leading_whitespace";
+	snprintf(status[i].value, MAXVALLEN, "%d", pool_config.insert_lock);
+	status[i].desc = "ignore leading white spaces";
+	i++;
+
 	status[i].name = "current_backend_host_name";
 	snprintf(status[i].value, MAXVALLEN, "%s", pool_config.current_backend_host_name);
 	status[i].desc = "current master host name";
@@ -2222,9 +2367,18 @@ static int load_balance_enabled(POOL_CONNECTION_POOL *backend, char *sql)
 	if (pool_config.load_balance_mode &&
 		DUAL_MODE &&
 		MAJOR(backend) == PROTO_MAJOR_V3 &&
-		TSTATE(backend) == 'I' &&
-		!strncasecmp(sql, "SELECT", 6))
-		return 1;
+		TSTATE(backend) == 'I')
+	{
+		if (pool_config.ignore_leading_white_space)
+		{
+			/* ignore leading white spaces */
+			while (*sql && isspace(*sql))
+				sql++;
+		}
+
+		if (!strncasecmp(sql, "SELECT", 6))
+			return 1;
+	}
 	return 0;
 }
 
